#!/bin/bash
#

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

set_compiler_options()
{
  if [[ "${OS1}" != "Darwin" ]] && \
    ver_greater 4.8.0 $(gcc -dumpversion); then
    # Find the GCC used to build the GCC we need
    gccprod=$(cat ${CLANG_FQ_DIR}/etc/gcc_dep 2>/dev/null || echo "gcc v6_4_0")
    eval setup -B ${gccprod} || ssi_die "Unable to set up ${gccprod} required for use of Clang SDK."
    (( want_self_contained_build = 1 ))
  fi
  cc=cc
  cxx=c++
}

unpack_and_patch()
{
    mkdir -p ${sourcedir}

    # unpack
    castxmltar=${tardir}/castxml-${origpkgdotver}.tar
    if [[ -d "${sourcedir}/castxml-${pkgdotver}" ]]; then
      echo "castxml source already exists."
    fi
    cd ${sourcedir}
    tar xf "${castxmltar}"
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
if [[ "${basequal}" == tar ]]; then
  maketar=${basequal}
  unset basequal
else
  maketar=${3}
fi

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=castxml
origpkgver=v0_4_2
origpkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgver=${origpkgver}
#pkgver=${origpkgver//-/_}_f20180122
pkgdotver=`echo ${pkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
ssibuildshims_version=v1_04_15
clang_version=v5_0_1
get_this_dir

get_ssibuildshims

if [[ -n "${basequal}" ]]; then
  bqspec=NO_TYPE
else
  bqspec=NO_QUAL
fi

source define_basics ${bqspec}

echo "building ${package} ${pkgver} ${fullqual:+-q ${fullqual}} for ${OS}-${plat} (flavor ${flvr})"

if [ "${maketar}" = "tar" ] && [ -d ${pkgdir}/bin ]
then
  # N.B. explicitly exclude source
  ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
    -d data:doc:include:scripts:example:share \
    ${product_dir} \
    ${package} \
    ${pkgver} \
    ${fullqual:-NO_QUAL}
   exit $?
fi

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi

[[ -d "${sourcedir}" ]] || unpack_and_patch

source setup_cmake

setup -z ${product_dir} -z ${PRODUCTS} -B clang ${clang_version} || \
  ssi_die "Unable to set up clang as required for build."

set_compiler_options

num_cores=$(${SSIBUILDSHIMS_DIR}/bin/ncores)

set -x

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual:-NO_QUAL}

setup -B ${package} ${pkgver} ${fullqual:+-q ${fullqual}} -z ${fakedb}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

cd ${blddir}
env CC=${cc} CXX=${cxx} \
  ${want_self_contained_build:+LDFLAGS="-static-libstdc++ -static-libgcc"} \
  cmake ../../source/castxml-${origpkgdotver} \
  -DCMAKE_BUILD_TYPE=Release \
  -DCMAKE_INSTALL_PREFIX=${pkgdir} \
  ${CLANG_FQ_DIR:+-DLLVM_DIR=${CLANG_FQ_DIR}/lib/cmake/llvm} || \
  ssi_die "Unable to configure castxml."

make -j "${num_cores}" || ssi_die "Unable to make castxml."
make -j "${num_cores}" install || ssi_die "Unable to install castxml."

set +x

if [ ! -d ${pkgdir}/bin ]
then
   echo "ERROR: failed to create ${pkgdir}/bin"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
"${SSIBUILDSHIMS_DIR}/bin/declare_product" "${product_dir}" ${package} ${pkgver} ${fullqual:-NO_QUAL}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver} ${fullqual:+-q ${fullqual}}"
source `ups setup -z "${product_dir}" -z "${PRODUCTS}" ${package} ${pkgver} ${fullqual:+-q ${fullqual}}`
echo "${package} ${pkgver} ${fullqual:+-q ${fullqual}} is installed at ${CASTXML_FQ_DIR}"

if [ "${maketar}" = "tar" ]; then
  # N.B. explicitly exclude source
  ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball \
    -d data:doc:include:scripts:example:share \
    ${product_dir} \
    ${package} \
    ${pkgver} \
    ${fullqual:-NO_QUAL}
fi

exit 0
